import { Color, CatmullRomCurve3, Vector3, Scene, Vector2 } from 'three'
import { useEffect } from '@storybook/client-api'

import { useThree } from '../setup'

import { GeometryUtils, Line2, LineMaterial, LineGeometry } from '../../src'

const Template = ({ dashed = false }) => {
  // must run in this so the canvas has mounted in the iframe & can be accessed by `three`
  useEffect(() => {
    const { scene, renderer } = useThree()

    // Position and THREE.Color Data

    const positions = []
    const colors = []

    const points = GeometryUtils.hilbert3D(new Vector3(0, 0, 0), 20.0, 1, 0, 1, 2, 3, 4, 5, 6, 7)

    const spline = new CatmullRomCurve3(points)
    const divisions = Math.round(12 * points.length)
    const point = new Vector3()
    const color = new Color()

    for (let i = 0, l = divisions; i < l; i++) {
      const t = i / l

      spline.getPoint(t, point)
      positions.push(point.x, point.y, point.z)

      color.setHSL(t, 1.0, 0.5)
      colors.push(color.r, color.g, color.b)
    }

    const geometry = new LineGeometry()
    geometry.setPositions(positions)
    geometry.setColors(colors)

    const matLine = new LineMaterial({
      color: 0xffffff,
      linewidth: 5, // in pixels
      vertexColors: true,
      resolution: new Vector2(window.innerWidth, window.innerHeight),
      dashed,
    })

    const line = new Line2(geometry, matLine)
    line.computeLineDistances()
    line.scale.set(1, 1, 1)

    scene.add(line)

    return () => {
      renderer.dispose()
    }
  }, [dashed])

  return ''
}

export const Default = Template.bind({})

export default {
  title: 'Geometry/Line',
  argTypes: {
    dashed: {
      name: 'dashed',
      control: {
        type: 'boolean',
      },
    },
  },
}
